/**
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.GWTConsole.client.Commons.celltable;

import com.google.gwt.core.client.GWT;
import com.google.gwt.resources.client.ImageResource;
import com.google.gwt.resources.client.ImageResource.ImageOptions;
import com.google.gwt.user.cellview.client.CellTable;
import com.google.gwt.user.cellview.client.Column;
import com.google.gwt.user.cellview.client.ColumnSortEvent.ListHandler;
import com.google.gwt.view.client.ProvidesKey;

import java.util.Comparator;

/**
 * @author Guillaume Porcher
 * 
 */
public class ConsoleCellTable<T> extends CellTable<T> {

  /**
   * @author Guillaume Porcher
   * 
   */
  private final class ColumnComparator<U extends Comparable<U>, C extends Column<T, U>> implements Comparator<T> {

    private final C column;

    /**
     * @param column
     */
    private ColumnComparator(final C column) {
      this.column = column;
    }
    public int compare(final T o1, final T o2) {
      if (o1 == o2) {
        return 0;
      }

      if (o1 == null && o2 != null) {
        return -1;
      } else if (o1 != null && o2 == null) {
        return 1;
      }
      // Compare the column values.
      if (this.column.getValue(o1) != null) {
        return (this.column.getValue(o2) != null) ? this.column.getValue(o1).compareTo(this.column.getValue(o2)) : 1;
      }
      return -1;
    }
  }

  public <U extends Comparable<U>> void addSortedColumn(
      final ListHandler<T> columnSortHandler, final Column<T, U> column, final String columnName) {
    // Make the column sortable.
    column.setSortable(true);
    columnSortHandler.setComparator(column, new ColumnComparator<U, Column<T, U>>(column));
    // Add the columns.
    this.addColumn(column, columnName);

  }

  private static Resources defaultResources;

  private static Resources getDefaultResources() {
    if (ConsoleCellTable.defaultResources == null) {
      ConsoleCellTable.defaultResources = GWT.create(ConsoleCellTableResources.class);
    }
    return ConsoleCellTable.defaultResources;
  }

  public interface ConsoleCellTableResources extends CellTable.Resources {

    /**
     * Icon used when a column is sorted in ascending order.
     */
    @Source("sortAscending.png")
    @ImageOptions(flipRtl = true)
    ImageResource cellTableSortAscending();

    /**
     * Icon used when a column is sorted in descending order.
     */
    @Source("sortDescending.png")
    @ImageOptions(flipRtl = true)
    ImageResource cellTableSortDescending();

    @Source({
        CellTable.Style.DEFAULT_CSS, "ConsoleCellTable.css"
    })
    ConsoleCellTableStyle cellTableStyle();
  }

  public interface ConsoleCellTableStyle extends CellTable.Style {
  }

  protected static final int DEFAULT_PAGESIZE = 15;

  public ConsoleCellTable() {
    super(ConsoleCellTable.DEFAULT_PAGESIZE, ConsoleCellTable.getDefaultResources());
    this.setKeyboardSelectionPolicy(KeyboardSelectionPolicy.DISABLED);
  }

  public ConsoleCellTable(final int pageSize, final ProvidesKey<T> keyProvider) {
    super(pageSize, ConsoleCellTable.getDefaultResources(), keyProvider);
    this.setKeyboardSelectionPolicy(KeyboardSelectionPolicy.DISABLED);
  }
}
