/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.ListInstanceActivity;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.shared.HandlerRegistration;
import org.ow2.orchestra.console.GWTConsole.client.Commons.PresenterIF;
import org.ow2.orchestra.console.GWTConsole.client.ConsoleEventBus;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshHandler;
import org.ow2.orchestra.console.GWTConsole.client.ListInstance.PListInstanceCellTable;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.ProcessDetails.PProcessViewerPanel;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;

/**
 * Presenter of the instance activity view
 *
 * @author Jean-Yves Goubet
 */
public class PListInstanceActivity extends PresenterIF {

  private VListInstanceActivityIF view;
  private String instanceId = null;

  private HandlerRegistration handlerRegistration;
  private final PListInstanceCellTable pListInstanceCellTable;
  private final PListInstanceActivitiesCellTable pListInstanceActivitiesCellTable;
  private final PProcessViewerPanel processViewHandler;

  protected OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);

  public PListInstanceActivity(final PTabLayoutPanel presenterTabLayoutPanel,
                               final String instanceId, final String instanceName) {
    //
    final VListInstanceActivity vListInstanceActivity = new VListInstanceActivity();
    this.bindingView(vListInstanceActivity);
    this.instanceId = instanceId;

    //
    presenterTabLayoutPanel.addTab(instanceId, instanceName, PTab.Type.LIST_INSTANCE_ACTIVITY, vListInstanceActivity, this);

    this.pListInstanceCellTable =
            new PListInstanceCellTable(
                    null,
                    instanceId,
                    presenterTabLayoutPanel,
                    this.view.getInstanceTable(),
                    "instanceDetailsCellTable-" + instanceId);

    this.processViewHandler = new PProcessViewerPanel(null, instanceId, PListInstanceActivity.this.view.getProcessViewer(), presenterTabLayoutPanel);

    this.pListInstanceActivitiesCellTable =
            new PListInstanceActivitiesCellTable(
                    instanceId,
                    presenterTabLayoutPanel,
                    this.view.getInstanceActivitiesTable(),
                    "instanceActivitiesCellTable-" + instanceId,
                    this.processViewHandler);

    this.view.getPager().setDisplay(this.view.getInstanceActivitiesTable());
    this.view.getPager().ensureDebugId("instanceActivitiesPager-" + instanceId);
    //
    this.addListeners();


    this.refresh();

  }

  @Override
  protected void bindingView(final Object view) {
    this.view = (VListInstanceActivityIF) view;
  }

  @Override
  protected void addListeners() {
    this.addViewRefreshHandlerToBus();
  }

  private void addViewRefreshHandlerToBus() {

    final ViewRefreshHandler refreshHandler = new ViewRefreshHandler() {

      public void onRefresh(final ViewRefreshEvent viewRefreshEvent) {
        if (PListInstanceActivity.this.instanceId.equals(viewRefreshEvent.getElementId())
            && viewRefreshEvent.getElementType() == PTab.Type.LIST_INSTANCE_ACTIVITY) {

          PListInstanceActivity.this.view.getInstanceTable().setVisibleRangeAndClearData(
                  PListInstanceActivity.this.view.getInstanceTable().getVisibleRange(), true);
          PListInstanceActivity.this.refresh();
        }
      }
    };

    this.handlerRegistration = ConsoleEventBus.getEventBus().addHandler(ViewRefreshEvent.TYPE, refreshHandler);
  }

  public void refresh() {
    this.pListInstanceActivitiesCellTable.refresh();
    // this.view.getInstanceTable().setVisibleRangeAndClearData(this.view.getInstanceTable().getVisibleRange(),
    // true);
    // this.pListInstanceCellTable.refresh();

  }

  @Override
  protected void setDebugID() {
    // no debug id
  }

  @Override
  public void destroy() {
    if (this.handlerRegistration != null) {
      this.handlerRegistration.removeHandler();
    }
  }
}
