/**
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.GWTConsole.client.ListProcessActivity;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.cellview.client.Column;
import com.google.gwt.user.cellview.client.ColumnSortEvent.ListHandler;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.view.client.ListDataProvider;
import java.util.List;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.ConsoleCellTable;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.LabelCell;
import org.ow2.orchestra.console.GWTConsole.client.ConsoleEventBus;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementException;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.Popup.PPopup;
import org.ow2.orchestra.console.GWTConsole.client.ProcessDetails.PProcessViewerPanel;


/**
 * @author Guillaume Porcher
 */
public class PProcessActivityCellTable {


  protected OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);

  // Create a data provider.
  private final ListDataProvider<MProcessActivity> dataProvider = new ListDataProvider<MProcessActivity>();

  private final String processId;
  private final String debugPrefix;

  private final PProcessViewerPanel pProcessViewerPanel;

  public PProcessActivityCellTable(final String processId, final ConsoleCellTable<MProcessActivity> table, final String debugPrefix, final PProcessViewerPanel pProcessViewerPanel) {
    this.processId = processId;
    this.debugPrefix = debugPrefix;

    table.ensureDebugId(debugPrefix);
    this.pProcessViewerPanel = pProcessViewerPanel;
    // Add a ColumnSortEvent.ListHandler to connect sorting to the
    // java.util.List.
    final ListHandler<MProcessActivity> columnSortHandler = new ListHandler<MProcessActivity>(this.dataProvider.getList());
    table.addColumnSortHandler(columnSortHandler);

    this.setColumns(table, columnSortHandler);

    // Connect the table to the data provider.
    this.dataProvider.addDataDisplay(table);

    // Fetch data
    this.refresh();

  }

  private void setColumns(final ConsoleCellTable<MProcessActivity> table, final ListHandler<MProcessActivity> columnSortHandler) {

    /* Name column */
    final Column<MProcessActivity, String> nameColumn = new Column<MProcessActivity, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MProcessActivity mprocess) {
        return mprocess.getProcessActivityName();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, nameColumn, "Name");

    /* Type column */
    final Column<MProcessActivity, String> typeColumn = new Column<MProcessActivity, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MProcessActivity mprocess) {
        return mprocess.getProcessActivityType();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, typeColumn, "Type");

    /* Documentation column */
    final Column<MProcessActivity, String> documentationColumn = new Column<MProcessActivity, String>(new LabelCell(true, false, this.debugPrefix)) {
      @Override
      public String getValue(final MProcessActivity mprocess) {
        return mprocess.getDocumentation();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, documentationColumn, "Documentation");
  }

  protected void refresh() {
    // Add the data to the data provider, which automatically pushes it to the
    // widget.
    this.orchestramanagement.getProcessActivities(this.processId, new AsyncCallback<List<MProcessActivity>>() {
      public void onSuccess(final List<MProcessActivity> mProcessActivityList) {
        PProcessActivityCellTable.this.dataProvider.getList().clear();
        PProcessActivityCellTable.this.dataProvider.getList().addAll(mProcessActivityList);

        PProcessActivityCellTable.this.pProcessViewerPanel.setProcessData(mProcessActivityList);

        ConsoleEventBus.getEventBus().fireEvent(new RefreshDoneEvent());
      }

      public void onFailure(final Throwable caught) {
        GWT.log("ProcessActivityGrid - getProcessActivities: onFailure : " + caught.getMessage());
        ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
        if (caught instanceof OrchestraManagementException) {
          PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                  ((OrchestraManagementException) caught).getDetails());
        } else {
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
        }
      }
    });
  }

}
